%% Calling Shared Libraries from Simulink
% This paper discusses several approaches for calling shared libraries from
% Simulink models.
%
% *NOTE:* This example is best viewed in MATLAB's built-in browser. Simply
% download this FileExchange submission and open the HTML file in MATLAB.
%
% Consider the example library called *exlib*. Its code is provided in
% files <matlab:edit('exlib.c') _exlib.c_> and <matlab:edit('exlib.h')
% _exlib.h_>. This is a very simple library that consists of three
% functions:
%
% *void exlib_init(void)* opens a file for writing.
%
% *void exlib_print(float data)* writes supplied data into the file.
%
% *void exlib_term(void)* closes the file.
%% 1. Compiling the example library
% To start off, let's compile the library.
%
% Obviously, you don't need this step if your library is precompiled and
% ships to you as a binary file. In this case you can go straight to step
% 3, but note that you need to get *.dll* (or *.so* for Linux) file and the
% according header file (*.h*) from your library provider. For Windows, you
% will also need the *.lib* file - which is not the static library, but the
% so called import library. You need the import library in case you are
% going to do
% <matlab:web('https://docs.microsoft.com/cpp/build/linking-implicitly','-browser')
% implicit linking>.
%
% We are using MATLAB <matlab:doc('mex') mex> command which in turn calls the
% currently active host compiler to compile the shared library (_exlib.dll_ on
% Windows, and _exlib.so_ on Linux). It is important to make sure that you have
% ran the
%
%   mex -setup
%
% command first, to choose the
% <matlab:web('http://www.mathworks.com/support/compilers/current_release/','-browser')
% supported host compiler>.
%%
% Little trick: Using mex to compile the shared library.
mingw = strfind(mex.getCompilerConfigurations('C','Selected').Name,'MinGW64 Compiler');
if isunix
    % GCC
    mex('LDEXT=.so','LINKEXPORT=','LINKEXPORTVER=','LINKLIBS=','exlib.c');
elseif mingw
    % MinGW builds static shared library, so dll and lib files are the same
    % loadlibrary uses the dll file, while legacy code tool looks for the lib file
    mex('LDEXT=.lib','LINKEXPORT=','LINKEXPORTVER=','LINKLIBS=','exlib.c');
    mex('LDEXT=.dll','LINKEXPORT=','LINKEXPORTVER=','LINKLIBS=','exlib.c');
else
    % Visual
    mex('LDEXT=.dll','LINKEXPORT=','LINKEXPORTVER=','CMDLINE300="del exlib.exp exlib.dll.manifest"','exlib.c');
end
%% 2. Making sure the library can be loaded
% We can test this shared library from MATLAB (though it's not a scope of
% this paper) to make sure it works.
% 
%   % Load the library
%   [~,~] = loadlibrary(['exlib',system_dependent('GetSharedLibExt')],'exlib.h');
%   % Display functions available in the library
%   libfunctionsview('exlib');
%   % Initialize
%   calllib('exlib','exlib_init');
%   % Step
%   for i = 1:10
%       calllib('exlib','exlib_print',single(i));
%   end
%   % Terminate
%   calllib('exlib','exlib_term');
%   % Unload the library
%   unloadlibrary('exlib');
%   % Show contents of generated file
%   type('exlib.txt');
%
% Now that the shared library is available, let's get it working with
% Simulink!
%% 3. Calling shared library using C S-function
% <matlab:web([docroot,'/simulink/c-c-s-functions.html']) C S-function> is
% a way to bring C code into Simulink. This approach allows user to write
% whatever C code is needed to load their library and call functions in it.
% This C code is then compiled into a binary file that is recognized by
% Simulink and linked against the shared library. This binary file is
% called S-function. There is an according block in Simulink to call this
% S-function. There are several approaches for creating of C S-functions in
% Simulink.
%%
% *3.1. Using
% <matlab:web([docroot,'/simulink/legacy-code-integration.html']) Legacy
% Code Tool>*
%
% One approach is to use Legacy Code Tool that helps users create
% S-functions automatically from supplied specifications expressed with
% MATLAB code. In this example, we link with import library (we need a
% *.lib file for that on Windows) and call C functions from that library.
% This approach is also called a load-time linking or implicit linking.
%%
% Initialize structure for Legacy Code Tool
specs = legacy_code('initialize');
% Prototype for the initialization function
specs.StartFcnSpec = 'exlib_init()';
% Prototype for the step function
specs.OutputFcnSpec = 'exlib_print(single u1)';
% Prototype for the terminate function
specs.TerminateFcnSpec = 'exlib_term()';
% Shared library to link with (.so on Linux and .dll on Windows)
specs.HostLibFiles = {['exlib',strrep(system_dependent('GetSharedLibExt'),'.dll','.lib')]};
% We must supply header file when linking with shared library, otherwise
% compiler might make wrong assumptions about function's prototype.
specs.HeaderFiles = {'exlib.h'};
specs.SFunctionName = 'sfun_exlib';
%%
% Create S-function, compile and link in one step:
legacy_code('generate_for_sim',specs);
%%
% Create Simulink block (if needed) using the generated S-function:
%
%   legacy_code('slblock_generate',specs);
%
%%
% Let's run the simulation:
open_system('simlib_test');
snapnow;
sim('simlib_test');
% Observe the results:
type('exlib.txt');
%%
% *3.2. Using
% <matlab:web([docroot,'/simulink/sfg/example-of-a-basic-c-mex-s-function.html'])
% manually written> S-Function*
%
% Another approach is to manually write our own S-Function to load shared
% library and call C functions from that library. With this approach, we will be
% using
% <matlab:web('https://docs.microsoft.com/cpp/build/linking-explicitly','-browser')
% explicit linking>, also referred to as run-time linking. The easiest is to
% adapt the S-Function automatically generated by Legacy Code Tool previously.
% In this example we will modify _mdlStart_, _mdlOutputs_ and _mdlTerminate_
% functions.
%
% Here is how these three functions look like after modification:
ctext = fileread('sfun_exlib_dyn.c');
match = regexp(ctext, 'void mdlStart.*?\n\}', 'match'); fprintf('%s\n\n',match{1});
match = regexp(ctext, 'void mdlOutputs.*?\n\}', 'match'); fprintf('%s\n\n',match{1});
match = regexp(ctext, 'void mdlTerminate.*?\n\}', 'match'); fprintf('%s',match{1});
%%
% Compile the manually written S-Function:
if isunix
    mex('sfun_exlib_dyn.c','-ldl');
else
    mex('sfun_exlib_dyn.c');
end
%%
% Let's run the simulation:
open_system('simlib_test_dyn');
snapnow;
sim('simlib_test_dyn');
% Observe the results:
type('exlib.txt');
%%
% *3.3. Using
% <matlab:web([docroot,'/simulink/sfg/building-s-functions-automatically.html'])
% S-Function Builder>*
%
% Yet another approach is to use S-Function Builder. This is a special
% block from Simulink library that in some sense stands between Legacy Code
% Tool and manually written S-Functions from complexity point of view.
% S-Function Builder provides a graphical user interface, where you
% describe characteristics of your S-Function and then it generates
% S-Function automatically for you.
%
% There are some limitations and performance issues when using S-Function
% Builder. This is considered now to be a pretty much outdated approach to
% creating S-Functions. If you are a prominent user of S-Function Builder, I
% encourage you to try out the C Function block, which appeared in R2020b and is
% discussed later on in thir paper.
%% 4. Calling shared library using MATLAB Function
% <matlab:web([docroot,'/simulink/slref/matlabfunction.html']) MATLAB
% Function> allows you to use MATLAB language to describe your algorithm in
% Simulink.
%
% In order to call shared library from MATLAB Function, we will be using
% <matlab:doc('coder.ceval') coder.ceval> function, which can only be used from
% within a MATLAB Function (and not MATLAB). You don't need to have
% <matlab:web('https://www.mathworks.com/products/matlab-coder.html','-browser')
% MATLAB Coder> installed for coder.ceval to work.
%
% This is how the MATLAB Function looks like:
%

% Display MATLAB Function
open_system('simlib_test_mlf');
open_system('simlib_test_mlf/MATLAB Function');
doc = matlab.desktop.editor.getActive;
disp(doc.Text);
close_system('simlib_test_mlf');
%%
% There is no good way to call terminate function when using MATLAB
% Function (compare to MATLAB System block below).
% 
% One way to define terminate function for the model is to put
% *exlib_term();* in Model Settings, Simulation Target -> Custom Code ->
% Terminate function.
%
% NOTE: If the
% <matlab:web(fullfile(docroot,'simulink/gui/parse-custom-code-symbols.html'))
% Import custom code> option is set, you have to specify all code dependencies
% in the Simulation Target panel (rather than using coder.cinclude and
% coder.updateBuildInfo). If this option is not set, you can combine Simulation
% Target settings, coder.cinclude and coder.updateBuildInfo.
% 
% Another way is to maintain the dependencies using coder.cinclude and
% coder.updateBuildInfo, and call *exlib_term()* conditionally in MATLAB
% Function. This is how I chose to do it in this example.
%%
% Let's run the simulation:
open_system('simlib_test_mlf');
snapnow;
sim('simlib_test_mlf');
% Observe the results:
type('exlib.txt');
%% 5. Calling shared library using Stateflow
% If you need to use your shared library functions in a
% <matlab:web('https://www.mathworks.com/products/stateflow.html','-browser'
% Stateflow> chart, it makes sense to call them directly from Stateflow. There
% are nice
% <matlab:web([docroot,'/stateflow/examples/including-custom-c-code-functions-and-structures.html'])
% examples> in Stateflow <matlab:web([docroot,'stateflow/custom-code.html'])
% documentation> that shows just how to do that.
%
% Calling external function in Stateflow is as easy as typing the
% function's name in the chart, when you want to use it:
%
open_system('simlib_test_sf');
snapnow;
%%
open_system('simlib_test_sf/Chart');
snapnow;
%%
% You need to adjust Model Settings so that Stateflow knows where to look
% for these external functions. In Simulation Target -> Custom Code ->
% Libraries pane, put *exlib.lib* (or *exlib.so* on Linux). In Simulation
% Target -> Custom Code -> Header File, put *#include "exlib.h"*. One
% potential pitfall is forgeting to include the terminate function. In
% Simulation Target -> Custom Code -> Terminate function, put
% *exlib_term();*.
%%
% Let's run the simulation:
if isunix
    set_param('simlib_test_sf','SimUserLibraries','exlib.so');
end
sim('simlib_test_sf');
% Observe the results:
type('exlib.txt');
%%
% Note that the information in this section is relevant for Stateflow
% charts that use <matlab:web([docroot,'stateflow/c-action-language.html'])
% C Action Language>. If you are using Stateflow with MATLAB Action
% Language, the approach is the same as with coder.ceval and MATLAB
% Function, described above.
%% 6. Calling shared library using MATLAB System block
% <matlab:web([docroot,'/simulink/slref/matlabsystem.html']) MATLAB System
% block> allows you to use System Object in Simulink. More information
% about this block can be found in
% <matlab:web([docroot,'/simulink/ug/what-is-matlab-system-block.html'])
% documentation>.
%
% Support for MATLAB System block appeared in Simulink in R2013b. Since
% then, I've seen many people using this approach, since it allows to easily
% define initialize/step/terminate functions. You can add in your own
% auxiliary MATLAB code around those functions - for example, to pre/post
% process data from step function - all without writing a single line of C
% code.
%
% Here is how System Object for use in MATLAB System block looks like:
%
% <include>exlib.m</include>
%%
% Let's run the simulation:
open_system('simlib_test_mls');
snapnow;
sim('simlib_test_mls');
% Observe the results:
type('exlib.txt');
%% 7. Calling shared library using C Caller block
% <matlab:web(fullfile(docroot,'simulink/slref/ccaller.html')) C Caller
% block> allows you to call C functions directly from Simulink. More
% information about this block can be found in
% <matlab:web(fullfile(docroot,'simulink/ug/integrate-ccode-ccaller.html'))
% documentation>.
%
% This is a relatively approach, which first appeared in MATLAB R2018b. Its
% main purpose is to make calls to C functions and libraries _ridiculously
% easy_ in Simulink. But it comes with some limitations that you can read
% about in the block's documentation.
%
% After I've added the *exlib.so/exlib.lib* to the *Simulation Target ->
% Libraries* pane and *#include "exlib.h"* to the *Simulation Target ->
% Header file* pane in model settings, it was just a matter of clicking
% "Refresh custom code" in C Caller block to be able to see all the
% functions in the library.
%
% After I've chosen the *exlib_print* function, port specification dialog
% was populated automatically:
%
% <<ccaller.png>>
%
% Don't forget to add calls to *exlib_init* and *exlib_term* functions
% using the same *Simulation Target* pane. Alternatively, drop in a couple
% of other C Caller blocks to call initialization and termination functions
% directly. If you choose to use C Caller blocks for init and term, you
% will probably find
% <matlab:web(fullfile(docroot,'simulink/slref/initializefunction.html'))
% Initialize Function> and
% <matlab:web(fullfile(docroot,'simulink/slref/terminatefunction.html'))
% Terminate Function> blocks very useful. You might also find this
% Stateflow example useful:
% <matlab:web(fullfile(docroot,'stateflow/ug/schedule-subsystems-to-execute-at-specific-times.html'))
% Schedule Subsystems to Execute at Specific Times>
%%
% Let's run the simulation:
open_system('simlib_test_ccaller');
snapnow;
if isunix
    set_param('simlib_test_ccaller','SimUserLibraries','exlib.so');
end
sim('simlib_test_ccaller');
% Observe the results:
type('exlib.txt');
%% 8. Calling shared library using C Function block
% The newest addition to dealing with integration of external code to Simulink
% is the <matlab:web(fullfile(docroot,'simulink/slref/cfunction.html')) C
% Function> block. It appeared in R2020a.
%
% It ressembles C Caller block in terms of ease of use, but allows to
% specify wrapper C code around imported functions (thus also ressembling
% S-Function Builder). I think the main use case for C Function block is,
% however, not about calling existing C functions, but rather writing small
% snippets of C code where deemed necessary in your application. For
% example, to access hardware registers or compiler intrinsics.
%
% I've added the *exlib.so/exlib.lib* to the *Simulation Target ->
% Libraries* pane and *#include "exlib.h"* to the *Simulation Target ->
% Header file* pane in model settings.
%
% After that, I've added an input symbol in C Function block with the *single*
% datatype and specified Output/Start/Terminate code like this:
%
% <<cfunction.png>>
%
%%
% Let's run the simulation:
open_system('simlib_test_cfunction');
snapnow;
if isunix
    set_param('simlib_test_cfunction','SimUserLibraries','exlib.so');
end
sim('simlib_test_cfunction');
% Observe the results:
type('exlib.txt');
%% 9. Calling shared library generated by Embedded Coder
% You can use
% <matlab:web('https://www.mathworks.com/products/embedded-coder.html','-browser')
% Embedded Coder> to automatically generate C code from your Simulink model
% and
% <matlab:web([docroot,'/ecoder/ug/creating-and-using-host-based-shared-libraries.html'])
% package it as a shared library>. There is also an
% <matlab:web([docroot,'/ecoder/examples/interfacing-to-host-simulators-using-shared-libraries.html'])
% example> in documentation that shows how to automatically generate shared
% library and call it from external application written in C.
%
% Note that if you just want to run your algorithm or part of your
% algorithm as a C code within the same Simulink model, then it's better
% done with Simulink Coder's
% <matlab:web([docroot,'/rtw/ug/generated-s-function-block-deployment.html'])
% S-Function target> or
% <matlab:web([docroot,'/ecoder/ug/about-sil-and-pil-simulations.html'])
% Software-in-the-Loop> simulation.
%
% However, if you already ship Embedded Coder generated shared library for
% example for HIL simulator, you might want to integrate the very same
% library into a larger model maintained by your collegue or customer (and
% protect the IP).
%
% Embedded Coder generated shared library is nothing different from the
% example shared library we used throughout the paper. Consider a simple
% model that has two inputs and one output:
open_system('simlib_test_ert');
snapnow;
%%
% After building the model, you will get .dll file (.so on Linux), .lib
% file (import library for the .dll) and .exp file (export file for linking
% with .dll).
if isunix
    set_param('simlib_test_ert','CustomHeaderCode','#include <stddef.h>');
end
rtwbuild('simlib_test_ert');
%%
% The generated shared library exports the following symbols:
buildDir = RTW.getBuildDir('simlib_test_ert').BuildDirectory;
cgDir = RTW.getBuildDir('simlib_test_ert').CodeGenFolder;
if isunix
    libName = 'simlib_test_ert';
else
    libName = ['simlib_test_ert_',computer('arch')];
end
fullLibName = fullfile(cgDir,[libName,system_dependent('GetSharedLibExt')]);
[~,~] = coder.loadlibrary(fullLibName,fullfile(buildDir,'simlib_test_ert.h'));
libfunctions(libName,'-full');
unloadlibrary(libName);
%%
% By default, inputs and outputs are exported as global symbols, and
% initialize/step/terminate functions follow the model naming convention.
% You can configure function prototypes, symbol names and storage if
% necessary (but that's way out of this paper's scope). In this model, i've
% set the model step function prototype to be _Out1 = ex_step(In1, In2)_.
%
% To call these functions you only have to apply one of the above methods.
% For example, I can use MATLAB Function (for simplicity, I only call the
% step function) to achieve that:
%

% Display MATLAB Function
open_system('simlib_test_callert');
open_system('simlib_test_callert/MATLAB Function');
doc = matlab.desktop.editor.getActive;
disp(doc.Text);
close_system('simlib_test_callert');
%%
% Let's simulate and observe results:
open_system('simlib_test_callert');
sim('simlib_test_callert');
snapnow;
%% 10. Conclusion
% This paper describes various approaches for calling shared libraries from
% Simulink models. Both implicit (load-time) and explicit (run-time)
% linking was considered. All methods have their pros and cons and their
% fitness depends on your specific workflow, requirements and aims.
%
% Legacy Code Tool approach is the best fit when implementing load-time
% linking with a shared library, since in this case we can just call
% functions from the library directly, and the linker will take care of the
% rest.
%
% MATLAB System block is another approach which provides the following
% advantages:
%
% * Fits very well in initialize/step/terminate paragidm, allowing to
% maintain all of these functions within the block itself and not
% model-wide
% * Allows to add custom code around the called external functions
% * Allows you to stay in MATLAB and write just MATLAB code
% * The MATLAB System block is self-contained and doesn't require
% additional steps to share (no need to compile the S-function, for
% example)
%
% The drawback of using MATLAB System block is that it might incur
% additional overhead at its interfaces during code generation. As of year
% 2019, Legacy Code Tool and S-functions are still preferred for production
% code generation.
%
% Manually written S-function is the best fit when implementing run-time
% linking with a shared library. In this case we have to use functions like
% _LoadLibrary / dlopen_, _GetProcAddress / dlsym_, _FreeLibrary / dlclose_
% to be able to call our functions.
%
% C Caller block, which appeared in R2018b, is by far the easiest way to
% call C functions, but it comes with its limitations.
%
% Same to say about C Function block, which is the newest addition in
% R2020a.